package dk.nsp.idsas.dao;

import dk.nsp.idsas.dao.entity.BlurringEntity;
import org.springframework.jdbc.core.BeanPropertyRowMapper;
import org.springframework.jdbc.core.namedparam.NamedParameterJdbcTemplate;

import javax.sql.DataSource;
import java.sql.Timestamp;
import java.time.*;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

public class BlurringDao {
    private final Period MAX_RETENTION;
    private final NamedParameterJdbcTemplate jdbcTemplate;
    private final BeanPropertyRowMapper<BlurringEntity> rowMapper;

    public BlurringDao(DataSource dataSource, Period maxRetention) {
        this.MAX_RETENTION = maxRetention;
        jdbcTemplate = new NamedParameterJdbcTemplate(dataSource);
        rowMapper = BeanPropertyRowMapper.newInstance(BlurringEntity.class);
    }

    public BlurringDao(DataSource dataSource) {
        this.MAX_RETENTION = Period.ofYears(5);
        jdbcTemplate = new NamedParameterJdbcTemplate(dataSource);
        rowMapper = BeanPropertyRowMapper.newInstance(BlurringEntity.class);
    }

    public void insert(BlurringEntity blurringEntity) {
        String query = "INSERT INTO blurring(patient_id, patient_id_type, organisation_id, organisation_id_type, expiry_date)" +
                "           VALUES (:patient_id, :patient_id_type, :organisation_id, :organisation_id_type, :expiry_date)";
        HashMap<String, Object> parameters = new HashMap<>();
        parameters.put("patient_id", blurringEntity.getPatientId());
        parameters.put("patient_id_type", blurringEntity.getPatientIdType());
        parameters.put("organisation_id", blurringEntity.getOrganisationId());
        parameters.put("organisation_id_type", blurringEntity.getOrganisationIdType());
        parameters.put("expiry_date", Timestamp.from(blurringEntity.getExpiryDate()));

        jdbcTemplate.update(query, parameters);
    }

    public List<BlurringEntity> getActiveBlurrings(String hashedPatientId, String patientIdType) {
        // Find de seneste sløringer, per patient/organisation, der ikke er udløbet
        String query =
                "SELECT b1.patient_id, b1.patient_id_type, b1.organisation_id, b1.organisation_id_type, b1.created_time, b1.expiry_date \n" +
                "FROM blurring b1\n" +
                "INNER JOIN (\n" +
                // Aggregering af 'max_created_time' ved at gruppere efter patient og organisation.
                "    SELECT patient_id, organisation_id, MAX(created_time) AS max_created_time\n" +
                "    FROM blurring\n" +
                "    WHERE patient_id = :patient_id AND patient_id_type = :patient_id_type\n" +
                "    GROUP BY patient_id, organisation_id\n" +
                ") b2 ON b1.patient_id = b2.patient_id AND b1.organisation_id = b2.organisation_id AND b1.created_time = b2.max_created_time\n" +
                "WHERE b1.patient_id = :patient_id\n" +
                "  AND b1.patient_id_type = :patient_id_type\n" +
                "  AND b1.expiry_date > NOW();";

        HashMap<String, Object> parameters = new HashMap<>();
        parameters.put("patient_id", hashedPatientId);
        parameters.put("patient_id_type", patientIdType);
        return jdbcTemplate.query(query, parameters, rowMapper);
    }

    public int getInactiveBlurringsCount() {
        final String query =
                "SELECT COUNT(*) FROM blurring " +
                "WHERE expiry_date <= :retention_time ";

        Map<String, Object> paramMap = new HashMap<>();
        // MySQL gør ikke det rigtige med en LocalDateTime, så vi laver det til en ISO-8601 date string
        paramMap.put("retention_time", LocalDateTime.now().minusYears(MAX_RETENTION.getYears()).toString());
        Integer count = jdbcTemplate.queryForObject(query, paramMap, Integer.class);
        return count == null ? 0 : count;
    }

    public int deleteRowsInBatch(int batchSize) {
        final String deleteQuery =
                "DELETE FROM blurring " +
                "      WHERE expiry_date <= :retention_time " +
                "      LIMIT :limit";

        Map<String, Object> paramMap = new HashMap<>();
        paramMap.put("limit", batchSize);
        // MySQL gør ikke det rigtige med en LocalDateTime, så vi laver det til en ISO-8601 date string
        paramMap.put("retention_time", LocalDateTime.now().minusYears(MAX_RETENTION.getYears()).toString());
        return jdbcTemplate.update(deleteQuery, paramMap);
    }

}
