package dk.nsp.idsas.security;

import dk.nsp.idsas.security.data.SecurityContextData;
import dk.nsp.idsas.security.data.TestUser;
import dk.nsp.idsas.security.impl.IdsasActorProviderImpl;
import dk.nsp.idsas.security.model.IdsasActor;
import dk.nsp.idsas.util.ErrorMessages;
import dk.sds.nsp.security.SecurityContext;
import org.junit.Assert;
import org.junit.Test;
import org.mockito.Mockito;

import java.time.ZonedDateTime;
import java.util.Optional;

import static junit.framework.TestCase.assertEquals;
import static org.junit.Assert.assertThrows;

public class IdsasActorProviderTest {

    @Test
    public void matchSystembruger() throws IdsasSecurityException {
        IdsasActorProvider provider = new IdsasActorProviderImpl(SecurityContextData::new);

        IdsasActor actor = provider.getActor();

        Assert.assertEquals(IdsasActor.IdsasActorType.SYSTEM, actor.getActorType());
        Assert.assertEquals(SecurityContextData.CVR, actor.getActorId());
        Assert.assertEquals(SecurityContextData.CLIENT_NAME, actor.getClientSystemName());
        Assert.assertEquals(SecurityContextData.CLIENT_KEY, actor.getClientKey());
    }

    @Test
    public void matchSundhedsfaglig() throws IdsasSecurityException {
        IdsasActorProvider provider = new IdsasActorProviderImpl(
                () -> new SecurityContextData() {
            @Override
            public Optional<User> getActingUser() {
                    return Optional.of(new TestUser() {
                        @Override
                        public UserType getUserType() {
                            return UserType.HealthcareProfessional;
                    }
                });
            }
        });
        IdsasActor actor = provider.getActor();
        Assert.assertEquals(actor.getActorType(), IdsasActor.IdsasActorType.SUNDHEDSFAGLIG);
    }

    @Test(expected = IdsasSecurityException.class)
    public void ingenMatchMedPrincipalUser() throws IdsasSecurityException {
        IdsasActorProvider provider = new IdsasActorProviderImpl(() -> new SecurityContextData() {
            @Override
            public Optional<User> getPrincipalUser() {
                return Optional.of(new TestUser());
            }
        });
        provider.getActor();
    }

    @Test(expected = IdsasSecurityException.class)
    public void ingenMatchMedActingUser() throws IdsasSecurityException {
        IdsasActorProvider provider = new IdsasActorProviderImpl(() -> new SecurityContextData() {
            @Override
            public Optional<User> getActingUser() {
                return Optional.of(new TestUser());
            }
        });
        provider.getActor();
    }

    @Test()
    public void sikkerhedsbillettenErIkkeGyldig() {
        // Arrange
        SecurityContext securityContext = Mockito.mock(SecurityContext.class);
        SecurityContext.Ticket ticket = Mockito.mock(SecurityContext.Ticket.class);
        Mockito.when(ticket.getValidTo()).thenReturn(Optional.of(ZonedDateTime.now().plusYears(1)));
        Mockito.when(ticket.getValidFrom()).thenReturn(Optional.of(ZonedDateTime.now().minusYears(1)));
        Mockito.when(securityContext.getTicket()).thenReturn(Optional.of(ticket));
        IdsasActorProvider provider = new IdsasActorProviderImpl(() -> securityContext);

        // Act
        IdsasSecurityException e = assertThrows(IdsasSecurityException.class, provider::getActor);

        // Assert
        String audience = securityContext.getTicket().flatMap(SecurityContext.Ticket::getAudience).orElse("");
        String created = securityContext.getTicket()
                .flatMap(SecurityContext.Ticket::getCreated)
                .map(Object::toString)
                .orElse("");
        assertEquals(ErrorMessages.ticketIsNotValidError(audience, created), e.getMessage());
    }

    @Test()
    public void sikkerhedsbillettenMåIkkeVæreUdløbet() {
        // Arrange
        SecurityContext securityContext = Mockito.mock(SecurityContext.class);
        SecurityContext.Ticket ticket = Mockito.mock(SecurityContext.Ticket.class);
        Mockito.when(ticket.getValidTo()).thenReturn(Optional.of(ZonedDateTime.now().minusDays(1)));
        Mockito.when(securityContext.getTicket()).thenReturn(Optional.of(ticket));
        IdsasActorProvider provider = new IdsasActorProviderImpl(() -> securityContext);

        // Act
        IdsasSecurityException e = assertThrows(IdsasSecurityException.class, provider::getActor);

        // Assert
        assertEquals(ErrorMessages.ticketIsExpiredError(), e.getMessage());
    }

    @Test()
    public void sikkerhedsbillettenMåIkkeFørstVæreGyldigSenere() {
        // Arrange
        SecurityContext securityContext = Mockito.mock(SecurityContext.class);
        SecurityContext.Ticket ticket = Mockito.mock(SecurityContext.Ticket.class);
        Mockito.when(ticket.getValidTo()).thenReturn(Optional.of(ZonedDateTime.now().plusYears(1)));
        Mockito.when(ticket.getValidFrom()).thenReturn(Optional.of(ZonedDateTime.now().plusDays(1)));
        Mockito.when(securityContext.getTicket()).thenReturn(Optional.of(ticket));
        IdsasActorProvider provider = new IdsasActorProviderImpl(() -> securityContext);

        // Act
        IdsasSecurityException e = assertThrows(IdsasSecurityException.class, provider::getActor);

        // Assert
        assertEquals(ErrorMessages.ticketIsNotYetValidError(), e.getMessage());
    }

    @Test()
    public void sikkerhedsbillettenMåIkkeMangle() {
        // Arrange
        SecurityContext securityContext = Mockito.mock(SecurityContext.class);
        Mockito.when(securityContext.getTicket()).thenReturn(Optional.empty());
        IdsasActorProvider provider = new IdsasActorProviderImpl(() -> securityContext);

        // Act
        IdsasSecurityException e = assertThrows(IdsasSecurityException.class, provider::getActor);

        // Assert
        assertEquals(ErrorMessages.ticketIsMissing(), e.getMessage());
    }
}
