package dk.nsp.idsas.service;

import dk.nsp.idsas.dao.BlurringDao;
import dk.nsp.idsas.dao.PatientIdSaltDao;
import dk.nsp.idsas.dao.WhitelistDao;
import dk.nsp.idsas.dao.entity.BlurringEntity;
import dk.nsp.idsas.integrations.personinformation.PersonInformationClient;
import dk.nsp.idsas.integrations.personinformation.PersonInformationClientException;
import dk.nsp.idsas.util.AuditLogger;
import dk.nsp.idsas.util.ErrorMessages;
import dk.nsp.idsas.util.HashingUtil;
import dk.nsp.idsas.util.WhitelistType;
import dk.sundhedsdatastyrelsen.identitetssloering._2023._06._01.PatientIdClassificationType;
import java.time.ZonedDateTime;

public class BlurringService {
    private final WhitelistDao whitelistDao;
    private final BlurringDao blurringDao;
    private final PatientIdSaltDao patientIdSaltDao;
    private final AuditLogger auditLogger;
    private final PersonInformationClient personInformationClient;

    public BlurringService(WhitelistDao whitelistDao, BlurringDao blurringDao, PatientIdSaltDao patientIdSaltDao,
                           AuditLogger auditLogger, PersonInformationClient personInformationClient) {
        this.whitelistDao = whitelistDao;
        this.blurringDao = blurringDao;
        this.patientIdSaltDao = patientIdSaltDao;
        this.auditLogger = auditLogger;
        this.personInformationClient = personInformationClient;
    }

    public void createBlurring(String actorId, String actorIdType, String patientId, String patientIdType, ZonedDateTime endDateTime) throws IdsasServiceException {

        // Tjek at actorId er whitelisted til at kalde createBlurring
        boolean isWhiteListed = whitelistDao.isWhitelisted(actorId, WhitelistType.BLURRING);
        if (!isWhiteListed) {
            throw new IdsasServiceException(ErrorMessages.cvrWhitelistError(actorId, WhitelistType.BLURRING), true);
        }

        // Tjek at patientId er gyldig (via "personinformation")
        if (patientIdType.equals(PatientIdClassificationType.CPR.value())) {
            boolean patientIdIsValid = false;
            try {
                patientIdIsValid = personInformationClient.personExists(patientId);
                if (!patientIdIsValid) {
                    throw new IdsasServiceException(ErrorMessages.cprIsNotValidError(patientId), true);
                }
            } catch (PersonInformationClientException e) {
                throw new IdsasServiceException(e.getMessage(), false);
            }
        }

        // Tjek at sløringen ikke ligger mere end 2 år fremme i tiden
        if (endDateTime.isAfter(ZonedDateTime.now().plusYears(2))) {
            throw new IdsasServiceException(ErrorMessages.maxExpiryDateExceeded(), true);
        }

        // PatientId hashes!
        byte[] patientIdSalt = patientIdSaltDao.getLatestSalt();
        String hashedPatientId = HashingUtil.createPatientIdHash(patientId, patientIdSalt);

        // Entity oprettes og gemmes
        BlurringEntity entity = new BlurringEntity(
                hashedPatientId,
                patientIdType,
                actorId,
                actorIdType,
                endDateTime.toInstant());

        blurringDao.insert(entity);

        // Audit logning
        auditLogger.logCreateBlurring(patientId, hashedPatientId, actorId);
    }

}
