package dk.nsp.idsas.service;

import dk.nsp.idsas.dao.BlurringDao;
import dk.nsp.idsas.dao.CleanupJobStatus;
import org.apache.log4j.Logger;

import java.time.Duration;
import java.util.concurrent.atomic.AtomicReference;

public class CleanupService {
    private static final Logger logger = Logger.getLogger(CleanupService.class);
    private final BlurringDao blurringDao;
    private final Duration maxDuration;
    private final int batchSize;
    private final int maxRetentionYears;
    private final AtomicReference<CleanupJobStatus> status = new AtomicReference<>();

    public CleanupService(BlurringDao blurringDao, Duration maxDuration, int batchSize, int maxRetentionYears) {
        this.blurringDao = blurringDao;
        this.maxDuration = maxDuration;
        this.batchSize = batchSize;
        this.maxRetentionYears = maxRetentionYears;
        this.status.set(CleanupJobStatus.notStarted());
    }

    public CleanupJobStatus getStatus() {
        return status.get();
    }

    public void cleanUpBlurrings() {
        logger.info(String.format("Starting cleanup job. Retention time: %s years", maxRetentionYears));
        try {
            // Hvor mange rækker skal slettes?
            int inactiveBlurringsCount = blurringDao.getInactiveBlurringsCount();

            // Start slettejob
            int totalRowsDeleted = doCleanUp(maxDuration, batchSize);

            // Udskriv status til loggen
            StringBuilder loginfo = new StringBuilder();
            loginfo.append("Cleanup job status:\n");
            loginfo.append(String.format("Rows deleted:         %10s\n", totalRowsDeleted));
            int rowsLeft = (inactiveBlurringsCount - totalRowsDeleted);
            loginfo.append(String.format("Rows left:            %10s\n", rowsLeft));
            if (rowsLeft > 0) {
                loginfo.append("Cleanup job did not finish. Please run again.\n");
            }
            status.set(new CleanupJobStatus(totalRowsDeleted, rowsLeft, rowsLeft <= 0 ?
                    CleanupJobStatus.ExecutionStatus.FINISHED : CleanupJobStatus.ExecutionStatus.NOT_FINISHED));
            loginfo.append("Cleanup job finished with status: ");
            loginfo.append(status.get().finished);

            logger.info(loginfo);
        } catch (Exception e) {
            logger.error(e.getMessage());
        }
    }

    private int doCleanUp(Duration maxDuration, int batchSize) {
        long startTime = System.currentTimeMillis();
        long elapsedMilliSeconds = 0;
        int totalRowsDeleted = 0;

        while (elapsedMilliSeconds < maxDuration.toMillis()) {
            int rowsDeleted = blurringDao.deleteRowsInBatch(batchSize);
            totalRowsDeleted += rowsDeleted;

            long currentTime = System.currentTimeMillis();
            elapsedMilliSeconds = currentTime - startTime;

            if (rowsDeleted < batchSize) {
                logger.trace("No more rows to delete.");
                break;
            }
        }

        return totalRowsDeleted;
    }

}
