package dk.nsp.idsas.service;

import dk.nsp.idsas.dao.BlurringDao;
import dk.nsp.idsas.dao.PatientIdSaltDao;
import dk.nsp.idsas.dao.WhitelistDao;
import dk.nsp.idsas.integrations.personinformation.PersonInformationClient;
import dk.nsp.idsas.integrations.personinformation.PersonInformationClientException;
import dk.nsp.idsas.security.model.IdsasActor;
import dk.nsp.idsas.util.AuditLogger;
import dk.nsp.idsas.util.ErrorMessages;
import dk.nsp.idsas.util.HashingUtil;
import dk.nsp.idsas.util.WhitelistType;
import org.junit.Before;
import org.junit.Test;
import org.mockito.Mockito;

import java.time.LocalDateTime;
import java.time.ZoneOffset;
import java.time.ZonedDateTime;
import java.util.Date;
import java.util.GregorianCalendar;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertThrows;

public class BlurringServiceTest {
    private WhitelistDao whitelistDao;
    private BlurringDao blurringDao;
    private PatientIdSaltDao patientIdSaltDao;
    private AuditLogger auditLogger;
    private PersonInformationClient personInformationClient;
    private BlurringService blurringService;

    @Before
    public void setUp() {
        whitelistDao = Mockito.mock(WhitelistDao.class);
        blurringDao = Mockito.mock(BlurringDao.class);
        patientIdSaltDao = Mockito.mock(PatientIdSaltDao.class);
        auditLogger = Mockito.mock(AuditLogger.class);
        personInformationClient = Mockito.mock(PersonInformationClient.class);
        blurringService = new BlurringService(whitelistDao, blurringDao, patientIdSaltDao, auditLogger, personInformationClient);
    }

    @Test
    public void createBlurring_WhenCvrIsNotWhitelisted_ThrowsException() {
        // Arrange
        IdsasActor actor = IdsasActor.createSystem("5678", "CVR", "TestSystem", "");
        Mockito.when(whitelistDao.isWhitelisted(actor.getActorId(), WhitelistType.BLURRING)).thenReturn(false);

        // Act
        IdsasServiceException e = assertThrows(IdsasServiceException.class,
                () -> blurringService.createBlurring(actor.getActorId(), actor.getActorIdType(), "", "", ZonedDateTime.now()));

        // Assert
        assertEquals(ErrorMessages.cvrWhitelistError(actor.getActorId(), WhitelistType.BLURRING), e.getMessage());
    }

    @Test
    public void createBlurring_WhenNoPersonInformationIsFoundForCpr_ThrowsException() throws PersonInformationClientException {
        // Arrange
        IdsasActor actor = IdsasActor.createSystem("5678", "CVR", "TestSystem", "");
        String cpr = "1111111111";
        Mockito.when(whitelistDao.isWhitelisted(actor.getActorId(), WhitelistType.BLURRING)).thenReturn(true);
        Mockito.when(personInformationClient.personExists(cpr)).thenReturn(false);

        // Act
        IdsasServiceException e = assertThrows(IdsasServiceException.class,
                () -> blurringService.createBlurring(actor.getActorId(), actor.getActorIdType(), cpr, "CPR", ZonedDateTime.now()));

        // Assert
        assertEquals(ErrorMessages.cprIsNotValidError(cpr), e.getMessage());
    }

    @Test
    public void createBlurring_WhenAllValidationsSucceed_CreatesBlurringAndAuditLogs() throws PersonInformationClientException, IdsasServiceException {
        byte[] patientIdSalt = {1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16};

        // Arrange
        IdsasActor actor = IdsasActor.createSystem("5678", "CVR", "TestSystem", "");
        String patientIdType = "CPR";
        String patientId = "1111111111";
        String hashedPatientId = HashingUtil.createPatientIdHash(patientId, patientIdSalt);
        ZonedDateTime endDateTime = ZonedDateTime.now();
        Mockito.when(whitelistDao.isWhitelisted(actor.getActorId(), WhitelistType.BLURRING)).thenReturn(true);
        Mockito.when(personInformationClient.personExists(patientId)).thenReturn(true);
        Mockito.when(patientIdSaltDao.getLatestSalt()).thenReturn(patientIdSalt);
        // Act
        blurringService.createBlurring(actor.getActorId(), actor.getActorIdType(), patientId, patientIdType, endDateTime);

        // Assert
        Mockito.verify(blurringDao).insert(Mockito.argThat(blurring -> {
            assertEquals(hashedPatientId, blurring.getPatientId());
            assertEquals(patientIdType, blurring.getPatientIdType());
            assertEquals(actor.getActorId(), blurring.getOrganisationId());
            assertEquals("CVR", blurring.getOrganisationIdType());
            assertEquals(endDateTime.toInstant(), blurring.getExpiryDate());
            return true;
        }));
        Mockito.verify(auditLogger).logCreateBlurring(patientId, hashedPatientId, actor.getActorId());
    }

    @Test
    public void createBlurring_WhenExpiryDateIsExceeded_ThrowsException() throws PersonInformationClientException {
        // Arrange
        IdsasActor actor = IdsasActor.createSystem("5678", "CVR", "TestSystem", "");
        ZonedDateTime exceededDate = ZonedDateTime.now().plusYears(2).plusDays(1);
        String patientIdType = "CPR";
        String patientId = "1111111111";
        Mockito.when(whitelistDao.isWhitelisted(actor.getActorId(), WhitelistType.BLURRING)).thenReturn(true);
        Mockito.when(personInformationClient.personExists(patientId)).thenReturn(true);

        // Act
        IdsasServiceException e = assertThrows(IdsasServiceException.class,
                () -> blurringService.createBlurring(actor.getActorId(), actor.getActorIdType(), patientId, patientIdType, exceededDate));

        // Assert
        assertEquals(ErrorMessages.maxExpiryDateExceeded(), e.getMessage());
    }

    private GregorianCalendar toGregorian(LocalDateTime datetime) {
        GregorianCalendar gregorianCalendar = (GregorianCalendar) GregorianCalendar.getInstance();
        gregorianCalendar.setTime(Date.from(datetime.toInstant(ZoneOffset.UTC)));
        return gregorianCalendar;
    }

}
